/*****************************************************************************/
/*                                     TLC.c                                 */
/*                             C File fr P-Steuerung                       */
/*                                  File TLC.c                               */
/*****************************************************************************/
#include <ADuC812.h>
#include <stdio.h>
#include "Menu.h"
#include "LCD.h"

// vordefinierde Werte
#define g 9.81;		// Gravi
#define v 0.8;		// Umrechnung mV->kg

// Leds und Tasten Festlegen
sbit P3_2=P3^2;
sbit P3_3=P3^3;
sbit P3_4=P3^4;
sbit RLED=P3^5;
sbit GLED=P3^6;
sbit FKTLED=P3^7;

// Funktionsprototypen
void init_system (void);
void init_data (void);
unsigned int getad (int ch);
void getkey (void);

// Anlegen der Menupointer
code void (*menu_pnt[])(void)=  {  M0,M1,M2 };

// Tabelle fr nchstes anzuzeigendes Men
code unsigned char nextmenu_tab [3][3]=
	{
	{ 1,2,0 },	// DOWN
	{ 2,0,1 },	// UP
	{ 0,1,2 },	// ENTER
	};

// Statischer Text
code char boot[42]="TLC by Bertram Schall & Michael Ornik 2003";
// Variablen
bit gedrueckt;
char taste,menu;
float last;
float ofs;

// Hauptprogramm
void main (void)
{
	unsigned int uofs,ulast;
	int i,j=0;	// Zwischenspeicher, Zhlvariable, ...
	char s[16];
	// Start des Bootvorganges
    RLED=0;
	// Initialisieren des LCD
	init_lcd();
	blank_lcd();
	// Initialisieren der Daten
	init_data();
	// Initialisieren der Hardware
	init_system();
	// Boot Text
	for(i=0;i<26;i++)
	{
		for(j=0;j<16;j++)
			char_lcd(1,j,boot[i+j]);
		warte(10);
	}
	blank_lcd();
	j=0;
	// Ende des Bootvorganges
	RLED=1;
	// Hauptmen Aktiv
	M0();
	// Hauptschleife
	while(1)
	{
		// Reset Watchdog
		WDR1=1;
		WDR2=1;
		// FKTLED blinken
		FKTLED^=0x01;
		// Feststellen ob Taste gedrueckt
		if((P3_2==0)||(P3_3==0)||(P3_4==0))
		gedrueckt=1;

		// nderung des akt. Menpunktes bzw. Menabh. Fkt?
		if(gedrueckt)
		{
			TR0=0;	// Stoppen des Timers fr Tasten
			gedrueckt=0;
			getkey();	// Liest in 'taste' den Tastenstatus ein
	       	menu=nextmenu_tab[taste][menu];
			blank_lcd();
			// Anzeigen des akt. Mens
	    	(*menu_pnt[menu])();

			// menpunktabhngige Funktionen
			switch(menu)
			{
				// Nullsetzten des Systems
				case 5 :{
							if(taste==2)
							{
								uofs=ulast;
								ofs=(ulast/1.638)-100.0;
							}
						 }						break;
				// Reset System
				case 10:{
							if(taste==2)
							{
								ofs=0.0;
							}
						 }						break;
				default: ;
			}	
			// min 500ms Wartezeit bis zum nchsten Menwechsel 
			warte(500);
			TR0=1;
		}

		// Dynamische Daten am LCD
		switch(menu)
		{
			// Anzeigen der akt. Kranlast
			case 0 :{
						// Display wird nur alle 5 HP Zyklen aktualisiert ->
						// bessere Lesbarkeit
						// Sender bekommt aber bei jedem Zyklus Daten (5x)
						j++;
						if(j==3)
						{
							j=0;
							ulast=getad(0);			// Messen
							last=ulast/1.638;		// Umwandeln in mV
							last=last-ofs;			// abzug von Offset
							last=last*v;			// in Kg
							last-=100;				// 100kg berlast von Messzelle gegen - Offset
							if(last>2000.0)			// Bei berlast Rote LED
								RLED=0;
							else
								RLED=1;
							sprintf(s,"%8.1f Kg",last);
							print_lcd(1,1,s);
						}
						// Telemetrie
						for(i=0;i<3;i++)
							printf("%s",s);
					 }						break;
			default: ;
		}
	}
}

// Initialisiert das System
void init_system (void)
{
	// Initialisieren der RS-232
	TCLK=1;
	RCLK=1;
	//fr 1200 Baud
	RCAP2L=-32;
	RCAP2H=-2;
	SCON=0x50;		// Mode 1, 8-bit UART, enable rcvr                
	TR2=1;			// Takterzeugung fr RS232 von T2
	TI=1;
	// Watchdog Timer 2048ms
	WDCON=0xE0;
	WDE=1;
	// Power Control fr Spannungsunterschreitung
	PSMCON=0x01;
	IE2=0x02;
	// Alle Int. Freigeben
	EA=1;
}

// Initialisiert die Daten
void init_data (void)
{
	gedrueckt=0;
	menu=0;
	taste=0;
	last=0.0;
	ofs=0.0;
}

// AD Wandler Funktion
unsigned int getad (int ch)
{
	char i;
	unsigned int ad=0;
	ADCCON1=124;
	ADCCON2=ch;								// Einstellen des Kanals
	for(i=0;i<50;i++)
	{
		SCONV=1;							// Starten der Wandlung
		while((ADCCON3>>8)==1)
				;							// Warte bis Fertig
		ad=ad+((ADCDATAH&0x0F)<<8)+ADCDATAL;
		if(i>0)
			ad=ad/2;						// mittel
	}
	return ad;
}

// Liest die gedrueckte Taste ein
void getkey (void)
{
	char i;
	// Mehrmalige Abfrage -> Tastenprellen
	for(i=0;i<15;i++)
	{
		warte(20);
		if(P3_2==0)
			taste=0;	// 1. Taste
		if(P3_3==0)
			taste=1;	// 2. Taste
		if(P3_4==0)
			taste=2;	// 3. Taste
	}
}

// PSM Interrupt -> Spannungsproblem
void PSM_ISR (void) interrupt 8
{
	WDE=0;	// Watchdog abschalten
	EA=0;	// Alle Interrupts Sperren
	RLED=0; // Led 1&2 einschalten
	GLED=0;
	// Benutzerinfo
	blank_lcd();
	print_lcd(1,1,"System schaltet");
	print_lcd(2,1,"in Leerlauf->BAT");
	warte(3000);
	blank_lcd();
	while(1)
	{
		FKTLED=1;	// FktLed aus
	}
}
